#R code for analyses in Martin, Montgomerie and Lougheed. Color patterns of closely-related bird species are more divergent at intermediate levels of breeding range sympatry. American Naturalist.

## xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
# RANKING ANALYSIS, Table 1, Figure 2A,B and Appendix Table S1 and Figure S1
## xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

dat <- read.csv(file.choose()) #use dialog box to import data file Appendix.4.csv and assign to dat

# define response variable 
dat$ranking <- cbind(dat$symp.diff, dat$allo.diff) #response is binomial variable

comparison <- factor(dat$number) #each comparison is numbered in the datafile; here converted to factor

#standardize predictors using arm package
library(arm)
dat$zoverlap <- rescale(dat$overlap)
dat$zgen.dist.A <- rescale(dat$gen.dist.A)
dat$zgen.dist.BC <- rescale(dat$gen.dist.BC)
dat$zoverlp.A <- rescale(dat$overlp.A)
dat$zoverlp.B <- rescale(dat$overlp.B)
dat$zmean.lat.ABC <- rescale(dat$mean.lat.ABC)
dat$zrange.size.B <- rescale(dat$range.size.B)
dat$zrange.size.C <- rescale(dat$range.size.C)

#test full model with comparison as random effect to avoid pseudoreplication
library(lme4)
# model 1
model1 <- glmer(ranking~zoverlap + zoverlp.A + zgen.dist.A + zgen.dist.BC + zoverlp.B + zmean.lat.ABC + zrange.size.B + zrange.size.C + continent.B + sex + (1|comparison), family = binomial, data = dat)
#model statistics; model failed to converge

#model 2, without random effect
model2 <- glm(ranking~zoverlap + zgen.dist.A + zgen.dist.BC + zoverlp.A + zoverlp.B + zmean.lat.ABC + zrange.size.B + zrange.size.C + continent.B + sex, family = binomial, data = dat)
summary(model2) #statistics show that this model is overdispersed; we do not report this model in the paper

# model 3, quasibinomial error distribution due to overdispersion of model2 (full model with binomial errors)
model3 <- glm(ranking~zoverlap + zgen.dist.A + zgen.dist.BC + zoverlp.A + zoverlp.B + zmean.lat.ABC + zrange.size.B + zrange.size.C + continent.B + sex, family = quasibinomial, data = dat)
summary(model3) #statistics for full model

# check diagnostics for full model
plot(model3) #diagnostic plots
shapiro.test(residuals(model3)) #test normality of residuals

# compare models with different predictors using information-theoretic approach with MuMIn package; rank models by delta QAICc
library(MuMIn)
dfun <- function(object) {
 with(object, sum((weights*residuals^2)[weights>0])/df.residual)
 } #function to extract the dispersion parameter, for use in dredge
model3.bin <- glm(ranking~zoverlap + zgen.dist.A + zgen.dist.BC + zoverlp.A + zoverlp.B + zmean.lat.ABC + zrange.size.B + zrange.size.C + continent.B + sex, family = binomial, na.action = "na.fail", data = dat) #binomial version of model2 needed for dredge function

# construct a function that allows for ML estimation - see see Bolker (2014) http://cran.r-project.org/web/packages/bbmle/vignettes/quasi.pdf
x.quasibinomial <- function(...) {
 res = quasibinomial(...)
 res$aic = binomial(...)$aic
 res
}

#model evaluation using Information Theoretic approach
model3.bin.2 <- update(model3.bin, family = "x.quasibinomial", na.action = "na.fail")
m3 <- dredge(model3.bin.2, rank = "QAICc", chat = dfun(model3.bin)) #chat (c-hat) uses dfun to extract the dispersion parameter from the global model, for use in calculating QAICc values; beta asks for standardized estimates
m3sub <- subset(m3, delta<2) #subset of models where delta QAICc <2
print(m3sub, abbrev = FALSE) #see this subset with full variable (column) names

#model averaging of top models (QAICc<2)
mod3avg.overdisp <- model.avg(m3, delta<2) # get averaged coefficients for overdispersed top models
summary(mod3avg.overdisp)
confint(mod3avg.overdisp) 

# model 4; best-fitting model, without standardizing predictors
model4 <- glm(ranking~overlap + overlp.B, family = quasibinomial, data = dat) # predictors not rescaled as actual slopes and intercepts to be interpreted
summary(model4) #model statistics reported in Table 1
confint(model4) #calculate 95%CL for model estimates
plot(model4) #plot model diagnstics
shapiro.test(residuals(model4)) #test if residuals normal

# predict ranking from model4, when overlp.B is 50%
lpmod4 <- predict.glm(model4, data.frame(overlap = 0, overlp.B = 0.5), se.fit = TRUE, dispersion = 6.289874) #note dispersion parameter needed to calculate SE correctly, taken from model 4 output
lpmod4 #print logit (predicted value) 
#calculate 95%CL for predicted value
qt(0.975, 100) #t-value for residual df
m4.hat <- predict.glm(model4, data.frame (overlap = 0, overlp.B = .5), se = TRUE, dispersion = 6.289874)
(lci <- m4.hat$fit - 1.983972*m4.hat$se.fit)
(uci <- m4.hat$fit + 1.983972*m4.hat$se.fit)

#convert logits to proportions
exp(lpmod4$fit)/((exp(lpmod4$fit) + 1)) #predicted value
exp(lci)/(exp(lci) + 1) #lower 95%CL
exp(uci)/(exp(uci) + 1) #lower 95%CL

#Figure2B data to plot
# predict ranking from model4, when overlp.B is 50%-100%
m4.hat <- predict.glm(model4, data.frame (overlap = 0, overlp.B = seq(.5, 1, by = .01)), se = TRUE, dispersion = 6.289874)
(lci <- m4.hat$fit - 1.983972*m4.hat$se.fit)
(uci <- m4.hat$fit + 1.983972*m4.hat$se.fit)
PoverlapB <- cbind(seq(.5, 1, by = .01))
dat2 <- data.frame(PoverlapB, lci, m4.hat$fit, uci) #create matrix of predicted values and CLs
write.table(dat2, file = "Fig2B.txt") #save values to file for plotting

## xxxxxxxxxxxxxxxxxxxxxxxx

# RATING ANALYSIS, Table 1, Figure 2C,D and Appendix Table S2, Figure S2

## xxxxxxxxxxxxxxxxxxxxxxxx

dat$GLR <- (log((dat$color.rating + sqrt(dat$color.rating^2 + 0.34052298842564^2))/2)-(-1.7395011087693))/1.00670876432162 #generalized logarithmic transformation of color.rating; from JMP 11

#standardize response variable using arm package
library(arm)
dat$zGLR <- rescale(dat$GLR)

#modelA using nlme
library(nlme)
modelA <- lme(zGLR~zoverlap + zgen.dist.A + zgen.dist.BC + zoverlp.A + zoverlp.B + zmean.lat.ABC + zrange.size.B + zrange.size.C + sex + continent.B, random = ~1|comparison, data = dat) #basic GLMM where GLrating is generalized logarithmic transform of color.rating, and all variables are standardized
qqnorm(resid(modelA)) #examine residuals from full model; see Figure S2
qqline(resid(modelA))
shapiro.test(residuals(modelA)) #test normality of residuals; see Figure S2

#evaluate models using Information Theoretic approach
library(MuMIn)
modA1 <- lme(zGLR~zoverlap + zgen.dist.A + zgen.dist.BC + zoverlp.A + zoverlp.B + zmean.lat.ABC + zrange.size.B + zrange.size.C + continent.B + sex, random = ~1|comparison, method = "ML", na.action = "na.fail", data = dat) #basic GLMM; ML method is necessary for model comparison using the IT approach 
mA <- dredge(modA1) #evaluate models by comparing AICc
print(subset(mA, delta<2), abbrev = FALSE) #list subset of models where delta AICc <2, with full variable (column) names; see Table S2

#model averaging based on top models in this set (AICc<2)
modAavg <- model.avg(mA, delta<2) # calculate averaged model
summary(modAavg) #statistics for averaged model
confint(modAavg) #95%CL of estimates in averaged model; see Table S2

#best-fitting model in this set, but variables not standardized here
#modelB
modelB <- lme(GLR~overlp.B + mean.lat.ABC + sex, random = ~1|comparison, data = dat) #REML model 
summary(modelB) #model statistics
intervals(modelB) #95%CLs for estimates
qqnorm(resid(modelB)) #examine residuals from model; see Figure S2
qqline(resid(modelB))
shapiro.test(resid(modelB)) #test normality of residuals; see Figure S2

# predict rating from modelB, when overlp.B is 50%
pRmodB <- predict(modelB, data.frame(overlp.B = 0.5, mean.lat.ABC = 0, sex = "both"), level = 0, se.fit = TRUE) #
pRmodB #print predicted value
#calculate 95%CL for predicted value
mB1.hat <- predict(modelB, data.frame (overlp.B = 0.5, mean.lat.ABC = 0, sex = "both"), level = 0, se.fit = TRUE)
(lci <- mB1.hat$fit - 1.983495*mB1.hat$se.fit)
(uci <- mB1.hat$fit + 1.983495*mB1.hat$se.fit)

#Figure 2D data for plotting
modelC <- lme(GLR~overlp.B + mean.lat.ABC, random = ~1|comparison, data = dat) #this is modelB without sex, to facilitate calculations
qt(0.975, 100) #t-value for df = 100
mC.hat <- predict(modelC, data.frame (overlp.B = seq(.5, 1, by = .01), mean.lat.ABC = 0), level = 0, se.fit = TRUE)
(lci <- mC.hat$fit - 1.983972*mC.hat$se.fit) #lower 95%CL
(uci <- mC.hat$fit + 1.983972*mC.hat$se.fit) #upper 95%CL

dat3 <- data.frame(PoverlapB, lci, mC.hat$fit, uci) #create dat frame of predicted values and CLs
write.table(dat3, file = "Fig2D.txt") #save values to file for plotting
